//
// hello.ADXL343.c
//
// ADXL343 accelerometer hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 11/8/15
// (c) Massachusetts Institute of Technology 2015
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define led_delay() _delay_ms(100) // LED delay

#define serial_port PORTB
#define serial_direction DDRB
#define serial_pin_out (1 << PA4)

#define I2C_slave_address 0x53 // ADXL345 alt address
#define I2C_delay() _delay_us(5)
#define SCL_pin (1 << PA1)
#define SCL_pins PINA
#define SCL_port PORTA
#define SCL_direction DDRA
#define SDA_pin (1 << PA0)
#define SDA_pins PINA
#define SDA_port PORTA
#define SDA_direction DDRA

#define led_port PORTA
#define led_direction DDRA
#define led_pin (1 << PA3)

void SCL_write(char bit) {
   //
   // write SCL bit
   //
   if (bit == 0) {
      output(SCL_direction,SCL_pin);
      clear(SCL_port,SCL_pin);
      }
   else {
      input(SCL_direction,SCL_pin);
      while (pin_test(SCL_pins,SCL_pin) == 0); // check for clock stretching
      }
   }

void SDA_write(char bit) {
   //
   // write SDA bit
   //
   if (bit == 0) {
      output(SDA_direction,SDA_pin);
      clear(SDA_port,SDA_pin);
      }
   else
      input(SDA_direction,SDA_pin);
   }

void I2C_init() {
   //
   // initialize I2C lines
   //
   SDA_write(1);
   SCL_write(1);
   } 

char I2C_master_write_byte(unsigned char byte) {
   //
   // master write I2C byte
   //
   unsigned char bit;
   //
   // loop over bits
   //
   for (bit = 0; bit < 8; ++bit) {
      if ((byte & 0x80) == 0)
         SDA_write(0);
      else
         SDA_write(1);
      SCL_write(1);
      I2C_delay();
      SCL_write(0);
      I2C_delay();
      byte <<= 1;
      }
   //
   // check for ACK
   //
   SDA_write(1);
   SCL_write(1);
   I2C_delay();
   if (pin_test(SDA_pins,SDA_pin) != 0) {
      //
      // no ACK, return 1
      //
      return 1;	
      }
   //
   // yes ACK, return 0
   //
   SCL_write(0);
   I2C_delay();
   return 0;
   }	

char I2C_master_write(unsigned char* data, unsigned char nbytes, unsigned char slave_address) {
   //
   // I2C master write
   //
   unsigned char index,ret,slave_address_write;
   //
   // send start
   //
   SDA_write(0);
   I2C_delay();
   SCL_write(0);	
   I2C_delay();
   //
   // send slave address
   //
   slave_address_write = slave_address << 1;
   if (I2C_master_write_byte(slave_address_write) != 0)
      //
      // no ACK, return 1
      //
      return 1;
   //
   // loop over bytes
   //
   for (index = 0; index < nbytes; ++index) {
      ret = I2C_master_write_byte(data[index]);
      if (ret != 0)
         //
         // no ACK, return 1
         //
         break;
      //
      // yes ACK, continue
      //	
      }
   //
   // send stop
   //
   SCL_write(1);
   I2C_delay();
   SDA_write(1);
   I2C_delay();
   return ret;
   }

void I2C_master_read_byte(unsigned char* data, unsigned char index, unsigned char nbytes) {
   //
   // master read I2C byte
   //
   unsigned char byte,bit;
   SDA_write(1);
   byte = 0;
   //
   // loop over bits
   //
   for (bit = 0; bit < 8; ++bit)  {
      SCL_write(1);
      I2C_delay();
      if (pin_test(SDA_pins,SDA_pin) != 0)
         byte |= (1 << (7-bit));
      SCL_write(0);
      I2C_delay();
      }
   data[index] = byte;
   if (index < (nbytes-1)) {
      //
      // not done, send ACK
      //
      SDA_write(0);
      SCL_write(1);
      I2C_delay();
      SCL_write(0);
      SDA_write(1);
      I2C_delay();
      }
   else {
      //
      // done, send NACK
      //
      SDA_write(1);
      SCL_write(1);
      I2C_delay();
      SCL_write(0);
      I2C_delay();
      }
   }	

char I2C_master_read(unsigned char* data, unsigned char nbytes, unsigned char slave_address) {
   //
   // I2C master read
   //
   unsigned char index,slave_address_read;
   //
   // send start
   //
   SDA_write(0);
   I2C_delay();
   SCL_write(0);	
   I2C_delay();
   //
   // send slave address
   //
   slave_address_read = (slave_address << 1) + 1;
   if (I2C_master_write_byte(slave_address_read) == 1)
      //
      // no ACK, return 1
      //
      return 1;
   //
   // loop over bytes
   //
   for (index = 0; index < nbytes; ++index)
      I2C_master_read_byte(data,index,nbytes);
   //
   // send stop
   //
   SCL_write(1);
   I2C_delay();
   SDA_write(1);
   I2C_delay();
   return 0;
   }	

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

int main(void) {
   //
   // main
   //
   unsigned char data[6];
   unsigned char ret;
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);

   // initialize LED pin
   //
   clear(led_port, led_pin);
   output(led_direction, led_pin);

   int WINDOW = 600;
   float EPS = 0.1;
   int32_t xfilt = 0;
   int32_t yfilt = 0;
   int32_t zfilt=0;

   //
   // main loop
   //

   I2C_init();
   data[0] = 0x2D; // POWER_CTL register
   data[1] = 8; // turn on measure bit
   ret = I2C_master_write(data,2,I2C_slave_address);
   while (1) {
      /*
      //
      // read device ID
      //
      data[0] = 0x00; // device ID register
      ret = I2C_master_write(data,1,I2C_slave_address);
      ret = I2C_master_read(data,1,I2C_slave_address);
      put_char(&serial_port,serial_pin_out,data[0]);
      */
      //
      // send framing
      //
      put_char(&serial_port,serial_pin_out,1);
      put_char(&serial_port,serial_pin_out,2);
      put_char(&serial_port,serial_pin_out,3);
      put_char(&serial_port,serial_pin_out,4);

       /*x0 = ord(ser.read())
   x1 = ord(ser.read())
   y0 = ord(ser.read())
   y1 = ord(ser.read())
   z0 = ord(ser.read())
   z1 = ord(ser.read())
   x = x0+255*x1
   if (0x8000 & x): 
      x = -(0x10000-x)
   xfilt = (1-EPS)*xfilt+EPS*x
   canvas.itemconfigure("xtext",text="X %.0f"%xfilt)
   xdisp = int(.2*WINDOW + WINDOW*(.35+.35*xfilt/1024.0))
   canvas.coords('xrect1',.2*WINDOW,.05*WINDOW,xdisp,.2*WINDOW)
   canvas.coords('xrect2',xdisp,.05*WINDOW,.9*WINDOW,.2*WINDOW)
   y = y0+255*y1
   if (0x8000 & y): 
      y = -(0x10000-y)
   yfilt = (1-EPS)*yfilt+EPS*y
   canvas.itemconfigure("ytext",text="Y %.0f"%yfilt)
   ydisp = int(.2*WINDOW + WINDOW*(.35+.35*yfilt/1024.0))
   canvas.coords('yrect1',.2*WINDOW,.3*WINDOW,ydisp,.45*WINDOW)
   canvas.coords('yrect2',ydisp,.3*WINDOW,.9*WINDOW,.45*WINDOW)
   z = z0+255*z1
   if (0x8000 & z): 
      z = -(0x10000-z)
   zfilt = (1-EPS)*zfilt+EPS*z
   canvas.itemconfigure("ztext",text="Z %.0f"%zfilt)
   zdisp = int(.2*WINDOW + WINDOW*(.35+.35*zfilt/1024.0))
   canvas.coords('zrect1',.2*WINDOW,.55*WINDOW,zdisp,.7*WINDOW)
   canvas.coords('zrect2',zdisp,.55*WINDOW,.9*WINDOW,.7*WINDOW)
   canvas.update()
   parent.after_idle(idle,parent,canvas)*/

      //
      // read and send data
      //
      data[0] = 0x32; // X0 register
      ret = I2C_master_write(data,1,I2C_slave_address);
      ret = I2C_master_read(data,6,I2C_slave_address);
      float x0 = data[0];
      float x1 = data[1];
      float y0 = data[2];
      float y1 = data[3];
      float z0 = data[4];
      float z1 = data[5];

      int32_t x = x0+255*x1;
      if (0x8000 & x){x = -(0x10000-x);}
      xfilt = (1-EPS)*xfilt+EPS*x;
      x = WINDOW*(.35+.35*xfilt/1024.0);

      int32_t y = y0+255*y1;
      if (0x8000 & y){y = -(0x10000-y);}
      yfilt = (1-EPS)*yfilt+EPS*y;
      y = WINDOW*(.35+.35*yfilt/1024.0);

      int32_t z = z0+255*z1;
      if (0x8000 & z){z = -(0x10000-z);}
      zfilt = (1-EPS)*zfilt+EPS*z;
      z = WINDOW*(.35+.35*zfilt/1024.0);

	if (abs(x) > 150){

      set(led_port, led_pin);
      led_delay();
}
   else {
      clear(led_port, led_pin);
      led_delay();

      
      }
   }
   }